﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using ThinkGeo.MapSuite.MvcEdition;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;

namespace DisplayASimpleMapForMvc.Controllers
{
    public class IndexController : Controller
    {
        //
        // GET: /DrawEditShapes/

        public ActionResult Index()
        {
            Map map = new Map("Map1",
               new System.Web.UI.WebControls.Unit(100, System.Web.UI.WebControls.UnitType.Percentage),
               new System.Web.UI.WebControls.Unit(100, System.Web.UI.WebControls.UnitType.Percentage));

            map.MapUnit = GeographyUnit.DecimalDegree;
            map.CurrentExtent = new RectangleShape(-125, 72, 50, -46);
            map.MapBackground = new BackgroundLayer(new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF")));
            map.CustomOverlays.Add(new WorldMapKitWmsWebOverlay());

            string pointShapeFilePath = Server.MapPath("~/Data/Point.shp");
            string lineShapeFilePath = Server.MapPath("~/Data/line.shp");

            // if there is no any shape files for saving, we should created a new one firstly. like the commented out code below.
            // Note: the dbf columns are optional. the reason we add a new column "UserName" is you can split different user according to this column.

            //Collection<DbfColumn> dbfCoumns = new Collection<DbfColumn>();
            //dbfCoumns.Add(new DbfColumn("UserName", DbfColumnType.String, 20, 0));
            //ShapeFileFeatureLayer.CreateShapeFile(ShapeFileType.Point, pointShapeFilePath, dbfCoumns, System.Text.Encoding.ASCII, OverwriteMode.Overwrite);
            //ShapeFileFeatureLayer.CreateShapeFile(ShapeFileType.Polyline, lineShapeFilePath, dbfCoumns, System.Text.Encoding.ASCII, OverwriteMode.Overwrite);

            ShapeFileFeatureLayer pointShapeFileFeatureLayer = new ShapeFileFeatureLayer(pointShapeFilePath,ShapeFileReadWriteMode.ReadWrite);
            pointShapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            pointShapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer lineShapeFileFeatureLayer = new ShapeFileFeatureLayer(lineShapeFilePath,ShapeFileReadWriteMode.ReadWrite);
            lineShapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Canal1;
            lineShapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay dynamicOverlay = new LayerOverlay("DynamicOverlay");
            dynamicOverlay.IsBaseOverlay = false;
            dynamicOverlay.Layers.Add("PointLayer",pointShapeFileFeatureLayer);
            dynamicOverlay.Layers.Add("LineLayer",lineShapeFileFeatureLayer);

            map.CustomOverlays.Add(dynamicOverlay);
            return View(map);
        }

        [MapActionFilter]
        public void SaveMap(Map map, GeoCollection<object> args)
        {
            Collection<Feature> features = MapHelper.ConvertJsonToFeatures(args[0].ToString());
            string currentUsername = args[1].ToString();

            foreach (Feature feature in features)
            {
                LayerOverlay dynamicOverlay = (LayerOverlay)map.CustomOverlays["DynamicOverlay"];
                ShapeFileFeatureLayer pointShapeFileFeatureLayer = (ShapeFileFeatureLayer)dynamicOverlay.Layers["PointLayer"];
                ShapeFileFeatureLayer lineShapeFileFeatureLayer = (ShapeFileFeatureLayer)dynamicOverlay.Layers["LineLayer"];

                feature.ColumnValues.Clear();
                feature.ColumnValues.Add("UserName",currentUsername);

                if (feature.GetWellKnownType() == WellKnownType.Point)
                {
                    pointShapeFileFeatureLayer.Open();
                    pointShapeFileFeatureLayer.FeatureSource.BeginTransaction();
                    pointShapeFileFeatureLayer.FeatureSource.AddFeature(feature);
                    pointShapeFileFeatureLayer.FeatureSource.CommitTransaction();
                }
                else if (feature.GetWellKnownType() == WellKnownType.Line)
                {
                    lineShapeFileFeatureLayer.Open();
                    lineShapeFileFeatureLayer.FeatureSource.BeginTransaction();
                    lineShapeFileFeatureLayer.FeatureSource.AddFeature(feature);
                    lineShapeFileFeatureLayer.FeatureSource.CommitTransaction();
                }
            }

            map.EditOverlay.Features.Clear();
        }
    }
}
